@@ -18,7 +18,7 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 
-use std::net::SocketAddr;
+use std::sync::atomic::AtomicBool;
 use std::sync::Arc;
 use std::time::Instant;
 
@@ -31,28 +31,53 @@ use nucliadb_node::grpc::middleware::{
 use nucliadb_node::grpc::writer::{NodeWriterEvent, NodeWriterGRPCDriver};
 use nucliadb_node::http_server::{run_http_server, ServerOptions};
 use nucliadb_node::node_metadata::NodeMetadata;
+use nucliadb_node::replication::replicator::connect_to_primary_and_replicate_forever;
+use nucliadb_node::replication::service::ReplicationServiceGRPCDriver;
+use nucliadb_node::replication::NodeRole;
 use nucliadb_node::settings::providers::env::EnvSettingsProvider;
 use nucliadb_node::settings::providers::SettingsProvider;
+use nucliadb_node::settings::Settings;
 use nucliadb_node::telemetry::init_telemetry;
 use nucliadb_node::{lifecycle, utils};
+use nucliadb_protos::replication;
 use tokio::signal::unix::SignalKind;
 use tokio::signal::{ctrl_c, unix};
-use tokio::sync::mpsc::UnboundedReceiver;
+use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
 use tonic::transport::Server;
-
 type GrpcServer = NodeWriterServer<NodeWriterGRPCDriver>;
+use nucliadb_node::shards::providers::unbounded_cache::AsyncUnboundedShardWriterCache;
+use tokio::sync::Notify;
 
 #[derive(Debug)]
 pub enum NodeUpdate {
     ShardCount(u64),
 }
 
+fn get_shutdown_notifier() -> (Arc<Notify>, Arc<AtomicBool>) {
+    // Returns a tuple with a shutdown notifier and a shutdown notified.
+    // This is useful when you want to notify a shutdown and wait for it to be notified
+    // but also need to be able to check if server is shutting down
+    //
+    let shutdown_notifier = Arc::new(Notify::new());
+    let shutdown_notified = Arc::new(AtomicBool::new(false));
+
+    let shutdown_notifier_clone = Arc::clone(&shutdown_notifier);
+    let shutdown_notified_clone = Arc::clone(&shutdown_notified);
+
+    tokio::spawn(async move {
+        shutdown_notifier_clone.notified().await;
+        shutdown_notified_clone.store(true, std::sync::atomic::Ordering::Relaxed);
+    });
+
+    (shutdown_notifier, shutdown_notified)
+}
+
 #[tokio::main]
 async fn main() -> NodeResult<()> {
     eprintln!("NucliaDB Writer Node starting...");
     let start_bootstrap = Instant::now();
 
-    let settings = Arc::new(EnvSettingsProvider::generate_settings()?);
+    let settings: Arc<Settings> = Arc::new(EnvSettingsProvider::generate_settings()?);
 
     let _guard = init_telemetry(&settings)?;
     let metrics = metrics::get_metrics();
@@ -66,20 +91,35 @@ async fn main() -> NodeResult<()> {
     lifecycle::initialize_writer(&data_path, &settings.shards_path())?;
     let node_metadata = NodeMetadata::new().await?;
     let (metadata_sender, metadata_receiver) = tokio::sync::mpsc::unbounded_channel();
-    let grpc_sender = metadata_sender.clone();
-    let writer_settings = Arc::clone(&settings);
-    let grpc_driver = NodeWriterGRPCDriver::new(writer_settings).with_sender(grpc_sender);
-    grpc_driver.initialize().await?;
 
     let host_key_path = settings.host_key_path();
-    let _ = utils::read_or_create_host_key(host_key_path)?;
+    let node_id = utils::read_or_create_host_key(host_key_path)?;
 
-    nucliadb_telemetry::sync::start_telemetry_loop();
+    nucliadb_node::analytics::sync::start_analytics_loop();
 
-    tokio::spawn(start_grpc_service(
-        grpc_driver,
-        settings.writer_listen_address(),
+    let (shutdown_notifier, shutdown_notified) = get_shutdown_notifier();
+    let shard_cache = Arc::new(AsyncUnboundedShardWriterCache::new(settings.shards_path()));
+
+    let mut replication_task = None;
+    if settings.node_role() == NodeRole::Secondary {
+        // when it's a secondary server, do not even run the writer GRPC service
+        // because nothing should happen through that interface
+        replication_task = Some(tokio::spawn(connect_to_primary_and_replicate_forever(
+            Arc::clone(&settings),
+            Arc::clone(&shard_cache),
+            node_id.to_string(),
+            Arc::clone(&shutdown_notified),
+        )));
+    }
+
+    let grpc_task = tokio::spawn(start_grpc_service(
+        Arc::clone(&settings),
+        Arc::clone(&shard_cache),
+        metadata_sender.clone(),
+        node_id.to_string(),
+        Arc::clone(&shutdown_notifier),
     ));
+
     {
         let task = update_node_metadata(metadata_receiver, node_metadata);
         match metrics.task_monitor("UpdateNodeMetadata".to_string()) {
@@ -95,18 +135,22 @@ async fn main() -> NodeResult<()> {
     }));
 
     info!("Bootstrap complete in: {:?}", start_bootstrap.elapsed());
-    eprintln!("Running");
 
-    wait_for_sigkill().await?;
+    wait_for_sigkill(Arc::clone(&shutdown_notifier)).await?;
 
     info!("Shutting down NucliaDB Writer Node...");
     metrics_task.abort();
+    grpc_task.await??; // wait for shutdown of server instead of aborting it
     let _ = metrics_task.await;
 
+    if let Some(repl_task) = replication_task {
+        repl_task.await??;
+    }
+
     Ok(())
 }
 
-async fn wait_for_sigkill() -> NodeResult<()> {
+async fn wait_for_sigkill(shutdown_notifier: Arc<Notify>) -> NodeResult<()> {
     let mut sigterm = unix::signal(SignalKind::terminate())?;
     let mut sigquit = unix::signal(SignalKind::quit())?;
 
@@ -116,11 +160,19 @@ async fn wait_for_sigkill() -> NodeResult<()> {
         _ = ctrl_c() => println!("Terminating on ctrl-c"),
     }
 
+    shutdown_notifier.notify_waiters();
+
     Ok(())
 }
 
-pub async fn start_grpc_service(grpc_driver: NodeWriterGRPCDriver, listen_address: SocketAddr) {
-    info!("Listening for gRPC requests at: {:?}", listen_address);
+pub async fn start_grpc_service(
+    settings: Arc<Settings>,
+    shard_cache: Arc<AsyncUnboundedShardWriterCache>,
+    metadata_sender: UnboundedSender<NodeWriterEvent>,
+    node_id: String,
+    shutdown_notifier: Arc<Notify>,
+) -> NodeResult<()> {
+    let listen_address = settings.writer_listen_address();
 
     let tracing_middleware = GrpcInstrumentorLayer;
     let debug_logs_middleware = GrpcDebugLogsLayer;
@@ -129,15 +181,34 @@ pub async fn start_grpc_service(grpc_driver: NodeWriterGRPCDriver, listen_addres
     let (mut health_reporter, health_service) = tonic_health::server::health_reporter();
     health_reporter.set_serving::<GrpcServer>().await;
 
-    Server::builder()
+    let mut server_builder = Server::builder()
         .layer(tracing_middleware)
         .layer(debug_logs_middleware)
         .layer(metrics_middleware)
-        .add_service(health_service)
-        .add_service(GrpcServer::new(grpc_driver))
-        .serve(listen_address)
+        .add_service(health_service);
+
+    if settings.node_role() == NodeRole::Primary {
+        let grpc_driver = NodeWriterGRPCDriver::new(Arc::clone(&settings), shard_cache.clone())
+            .with_sender(metadata_sender);
+        grpc_driver.initialize().await?;
+        let replication_server =
+            replication::replication_service_server::ReplicationServiceServer::new(
+                ReplicationServiceGRPCDriver::new(settings.clone(), shard_cache.clone(), node_id),
+            );
+        server_builder = server_builder
+            .add_service(GrpcServer::new(grpc_driver))
+            .add_service(replication_server);
+    }
+
+    eprintln!("Listening for gRPC requests at: {:?}", listen_address);
+    server_builder
+        .serve_with_shutdown(listen_address, async {
+            shutdown_notifier.notified().await;
+        })
         .await
         .expect("Error starting gRPC service");
+
+    Ok(())
 }
 
 pub async fn update_node_metadata(
