@@ -28,12 +28,11 @@ use nucliadb_core::protos::{
 };
 use nucliadb_core::tracing::{self, Span, *};
 use nucliadb_core::NodeResult;
-use nucliadb_telemetry::payload::TelemetryEvent;
-use nucliadb_telemetry::sync::send_telemetry_event;
-use tokio::fs;
 use tokio::sync::mpsc::UnboundedSender;
 use tonic::{Request, Response, Status};
 
+use crate::analytics::payload::AnalyticsEvent;
+use crate::analytics::sync::send_analytics_event;
 use crate::settings::Settings;
 use crate::shards::errors::ShardNotFoundError;
 use crate::shards::metadata::ShardMetadata;
@@ -41,20 +40,10 @@ use crate::shards::providers::unbounded_cache::AsyncUnboundedShardWriterCache;
 use crate::shards::providers::AsyncShardWriterProvider;
 use crate::shards::writer::ShardWriter;
 use crate::telemetry::run_with_telemetry;
-
-fn create_merge_task(shard: Arc<ShardWriter>) -> impl FnOnce() + Send + 'static {
-    let span = Span::current();
-    let info = info_span!(parent: &span, "merge task");
-    let merge_task = move || {
-        if let Err(err) = shard.merge() {
-            info!("A merge was attempted, but failed: {err:?}")
-        }
-    };
-    move || run_with_telemetry(info, merge_task)
-}
+use crate::utils::list_shards;
 
 pub struct NodeWriterGRPCDriver {
-    shards: AsyncUnboundedShardWriterCache,
+    shards: Arc<AsyncUnboundedShardWriterCache>,
     sender: Option<UnboundedSender<NodeWriterEvent>>,
     settings: Arc<Settings>,
 }
@@ -66,11 +55,10 @@ pub enum NodeWriterEvent {
 }
 
 impl NodeWriterGRPCDriver {
-    pub fn new(settings: Arc<Settings>) -> Self {
-        let shards = AsyncUnboundedShardWriterCache::new(settings.shards_path());
+    pub fn new(settings: Arc<Settings>, shard_cache: Arc<AsyncUnboundedShardWriterCache>) -> Self {
         Self {
-            shards,
             settings,
+            shards: shard_cache,
             sender: None,
         }
     }
@@ -120,7 +108,7 @@ impl NodeWriter for NodeWriterGRPCDriver {
         &self,
         request: Request<NewShardRequest>,
     ) -> Result<Response<ShardCreated>, Status> {
-        send_telemetry_event(TelemetryEvent::Create).await;
+        send_analytics_event(AnalyticsEvent::Create).await;
         let request = request.into_inner();
         let metadata = ShardMetadata::from(request);
         let new_shard = self.shards.create(metadata).await;
@@ -140,7 +128,7 @@ impl NodeWriter for NodeWriterGRPCDriver {
     }
 
     async fn delete_shard(&self, request: Request<ShardId>) -> Result<Response<ShardId>, Status> {
-        send_telemetry_event(TelemetryEvent::Delete).await;
+        send_analytics_event(AnalyticsEvent::Delete).await;
         // Deletion does not require for the shard to be loaded.
         let shard_id = request.into_inner();
         let deleted = self.shards.delete(shard_id.id.clone()).await;
@@ -173,16 +161,11 @@ impl NodeWriter for NodeWriterGRPCDriver {
         &self,
         _request: Request<EmptyQuery>,
     ) -> Result<Response<ShardIds>, Status> {
-        let mut entries = fs::read_dir(self.shards.shards_path.clone()).await?;
-        let mut shard_ids = Vec::new();
-        while let Some(entry) = entries.next_entry().await? {
-            let entry_path = entry.path();
-            if entry_path.is_dir() {
-                if let Some(id) = entry_path.file_name().map(|s| s.to_str().map(String::from)) {
-                    shard_ids.push(ShardId { id: id.unwrap() });
-                }
-            }
-        }
+        let shard_ids = list_shards(self.shards.shards_path.clone())
+            .await
+            .into_iter()
+            .map(|id| ShardId { id })
+            .collect();
 
         Ok(tonic::Response::new(ShardIds { ids: shard_ids }))
     }
@@ -194,7 +177,6 @@ impl NodeWriter for NodeWriterGRPCDriver {
         let shard_id = resource.shard_id.clone();
         let shard = self.obtain_shard(&shard_id).await?;
         let info = info_span!(parent: &span, "set resource");
-        let merge_task = create_merge_task(Arc::clone(&shard));
         let write_task = || {
             run_with_telemetry(info, move || {
                 shard
@@ -209,7 +191,6 @@ impl NodeWriter for NodeWriterGRPCDriver {
             })?;
         match status {
             Ok(mut status) => {
-                tokio::task::spawn_blocking(merge_task);
                 status.status = 0;
                 status.detail = "Success!".to_string();
                 Ok(tonic::Response::new(status))
@@ -236,7 +217,6 @@ impl NodeWriter for NodeWriterGRPCDriver {
         let shard_id = resource.shard_id.clone();
         let shard = self.obtain_shard(&shard_id).await?;
         let info = info_span!(parent: &span, "remove resource");
-        let merge_task = create_merge_task(shard.clone());
         let write_task = || {
             run_with_telemetry(info, move || {
                 shard
@@ -251,7 +231,6 @@ impl NodeWriter for NodeWriterGRPCDriver {
             })?;
         match status {
             Ok(mut status) => {
-                tokio::task::spawn_blocking(merge_task);
                 status.status = 0;
                 status.detail = "Success!".to_string();
                 Ok(tonic::Response::new(status))
@@ -443,7 +422,7 @@ impl NodeWriter for NodeWriterGRPCDriver {
     }
 
     async fn gc(&self, request: Request<ShardId>) -> Result<Response<EmptyResponse>, Status> {
-        send_telemetry_event(TelemetryEvent::GarbageCollect).await;
+        send_analytics_event(AnalyticsEvent::GarbageCollect).await;
         let shard_id = request.into_inner();
         let shard = self.obtain_shard(&shard_id.id).await?;
         let span = Span::current();
