@@ -29,10 +29,12 @@
 //! a distributed fashion. The API allows building other interfaces, as the
 //! already built PyO3 bindings.
 
+pub mod analytics;
 pub mod grpc;
 pub mod http_server;
 pub mod lifecycle;
 pub mod node_metadata;
+pub mod replication;
 pub mod settings;
 pub mod shards;
 pub mod telemetry;
