@@ -35,6 +35,7 @@ use crate::shards::writer::ShardWriter;
 use crate::shards::ShardId;
 use crate::{disk_structure, env};
 
+#[derive(Default)]
 pub struct AsyncUnboundedShardWriterCache {
     cache: RwLock<HashMap<ShardId, Arc<ShardWriter>>>,
     pub shards_path: PathBuf,
@@ -43,25 +44,30 @@ pub struct AsyncUnboundedShardWriterCache {
 impl AsyncUnboundedShardWriterCache {
     pub fn new(shards_path: PathBuf) -> Self {
         Self {
-            shards_path,
             // NOTE: as it's not probable all shards will be written, we don't
             // assign any initial capacity to the HashMap under the
             // consideration a resize blocking is not performance critical while
             // writting.
             cache: RwLock::new(HashMap::new()),
+            shards_path,
         }
     }
 }
 
 #[async_trait]
 impl AsyncShardWriterProvider for AsyncUnboundedShardWriterCache {
-    async fn create(&self, metadata: ShardMetadata) -> NodeResult<ShardWriter> {
-        let shard_id = Uuid::new_v4().to_string();
+    async fn create(&self, metadata: ShardMetadata) -> NodeResult<Arc<ShardWriter>> {
+        let shard_id = metadata.id.clone().unwrap_or(Uuid::new_v4().to_string());
+        let shard_id_moved = shard_id.clone();
         let shard_path = disk_structure::shard_path_by_id(&self.shards_path.clone(), &shard_id);
-        let new_shard =
-            tokio::task::spawn_blocking(move || ShardWriter::new(shard_id, &shard_path, metadata))
-                .await
-                .context("Blocking task panicked")??;
+        let new_shard = Arc::new(
+            tokio::task::spawn_blocking(move || {
+                ShardWriter::new(shard_id_moved, &shard_path, metadata)
+            })
+            .await
+            .context("Blocking task panicked")??,
+        );
+        self.cache.write().await.insert(shard_id, new_shard.clone());
         Ok(new_shard)
     }
 
@@ -90,7 +96,8 @@ impl AsyncShardWriterProvider for AsyncUnboundedShardWriterCache {
         .context("Blocking task panicked")??;
 
         let shard = Arc::new(shard);
-        cache.insert(shard_key, Arc::clone(&shard));
+        let cache_shard = Arc::clone(&shard);
+        cache.insert(shard_key, cache_shard);
         Ok(shard)
     }
 
