@@ -16,8 +16,8 @@
 // You should have received a copy of the GNU Affero General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+use std::collections::HashMap;
 use std::path::{Path, PathBuf};
-use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::shard_created::{
@@ -26,24 +26,21 @@ use nucliadb_core::protos::shard_created::{
 use nucliadb_core::protos::{
     DeleteGraphNodes, JoinGraph, OpStatus, Resource, ResourceId, VectorSetId, VectorSimilarity,
 };
-use nucliadb_core::thread;
 use nucliadb_core::tracing::{self, *};
+use nucliadb_core::{thread, IndexFiles};
 use nucliadb_procs::measure;
+use tokio::sync::Mutex;
 
 use crate::disk_structure::*;
-use crate::shards::metadata::ShardMetadata;
+use crate::shards::metadata::{ShardMetadata, Similarity};
 use crate::shards::versions::Versions;
 use crate::telemetry::run_with_telemetry;
 
-const MERGE_THRESHOLD: usize = 5;
-
 #[derive(Debug)]
 pub struct ShardWriter {
     pub metadata: ShardMetadata,
     pub id: String,
     pub path: PathBuf,
-    merging_available: AtomicBool,
-    writes_without_merge: AtomicUsize,
     text_writer: TextsWriterPointer,
     paragraph_writer: ParagraphsWriterPointer,
     vector_writer: VectorsWriterPointer,
@@ -52,28 +49,8 @@ pub struct ShardWriter {
     paragraph_service_version: i32,
     vector_service_version: i32,
     relation_service_version: i32,
-}
-
-impl PartialOrd for ShardWriter {
-    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        self.id.partial_cmp(&other.id)
-    }
-}
-impl PartialEq for ShardWriter {
-    fn eq(&self, other: &Self) -> bool {
-        self.id.eq(&other.id)
-    }
-}
-impl Ord for ShardWriter {
-    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
-        self.id.cmp(&other.id)
-    }
-}
-impl Eq for ShardWriter {}
-impl std::hash::Hash for ShardWriter {
-    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
-        self.id.hash(state)
-    }
+    pub gc_lock: Mutex<()>, // lock to be able to do GC or not
+    write_lock: Mutex<()>,  // be able to lock writes on the shard
 }
 
 impl ShardWriter {
@@ -127,12 +104,12 @@ impl ShardWriter {
             paragraph_writer: paragraphs.unwrap(),
             vector_writer: vectors.unwrap(),
             relation_writer: relations.unwrap(),
-            merging_available: AtomicBool::new(true),
-            writes_without_merge: AtomicUsize::new(0),
             document_service_version: versions.version_texts() as i32,
             paragraph_service_version: versions.version_paragraphs() as i32,
             vector_service_version: versions.version_vectors() as i32,
             relation_service_version: versions.version_relations() as i32,
+            gc_lock: Mutex::new(()),
+            write_lock: Mutex::new(()),
         })
     }
 
@@ -308,6 +285,8 @@ impl ShardWriter {
         let mut paragraph_result = Ok(());
         let mut vector_result = Ok(());
         let mut relation_result = Ok(());
+
+        let _lock = self.write_lock.blocking_lock();
         thread::scope(|s| {
             s.spawn(|_| text_result = text_task());
             s.spawn(|_| paragraph_result = paragraph_task());
@@ -319,7 +298,7 @@ impl ShardWriter {
         paragraph_result?;
         vector_result?;
         relation_result?;
-        self.writes_without_merge.fetch_add(1, Ordering::SeqCst);
+        self.new_generation_id(); // VERY NAIVE, SHOULD BE DONE AFTER MERGE AS WELL
         Ok(())
     }
 
@@ -366,6 +345,8 @@ impl ShardWriter {
         let mut paragraph_result = Ok(());
         let mut vector_result = Ok(());
         let mut relation_result = Ok(());
+
+        let _lock = self.write_lock.blocking_lock();
         thread::scope(|s| {
             s.spawn(|_| text_result = text_task());
             s.spawn(|_| paragraph_result = paragraph_task());
@@ -377,7 +358,9 @@ impl ShardWriter {
         paragraph_result?;
         vector_result?;
         relation_result?;
-        self.writes_without_merge.fetch_add(1, Ordering::SeqCst);
+
+        self.new_generation_id();
+
         Ok(())
     }
 
@@ -439,6 +422,9 @@ impl ShardWriter {
     ) -> NodeResult<()> {
         let mut writer = vector_write(&self.vector_writer);
         writer.add_vectorset(setid, similarity)?;
+
+        self.new_generation_id();
+
         Ok(())
     }
 
@@ -446,6 +432,9 @@ impl ShardWriter {
     pub fn remove_vectorset(&self, setid: &VectorSetId) -> NodeResult<()> {
         let mut writer = vector_write(&self.vector_writer);
         writer.remove_vectorset(setid)?;
+
+        self.new_generation_id();
+
         Ok(())
     }
 
@@ -479,45 +468,84 @@ impl ShardWriter {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn merge(&self) -> NodeResult<()> {
-        if !self.merging_available.swap(false, Ordering::SeqCst) {
-            return Ok(());
-        }
-
-        // Critical section, only one merge at a time
-
-        let current_writes = self.writes_without_merge.swap(0, Ordering::SeqCst);
-        let merge_result = if current_writes >= MERGE_THRESHOLD {
-            self.inner_merge()
-        } else {
-            self.writes_without_merge
-                .fetch_add(current_writes, Ordering::SeqCst);
-            Ok(())
-        };
-
-        // Merging is available again
-        self.merging_available.store(true, Ordering::SeqCst);
-        merge_result
-    }
-
-    fn inner_merge(&self) -> NodeResult<()> {
-        // TODO: add relations merge when it becomes a Tantivy only index.
-        paragraph_write(&self.paragraph_writer).merge()?;
-        text_write(&self.text_writer).merge()?;
-        vector_write(&self.vector_writer).merge()?;
-
-        // After merging GC is applied
-        // TODO: add relations gc when it becomes a Tantivy only index.
-        paragraph_write(&self.paragraph_writer).garbage_collection()?;
-        text_write(&self.text_writer).garbage_collection()?;
-        vector_write(&self.vector_writer).garbage_collection()?;
-
-        Ok(())
+    pub fn gc(&self) -> NodeResult<()> {
+        let _lock = self.gc_lock.blocking_lock();
+        vector_write(&self.vector_writer).garbage_collection()
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn gc(&self) -> NodeResult<()> {
-        // TODO: remove this operation from the interface
-        Ok(())
+    pub fn get_generation_id(&self) -> String {
+        // we can not cache this on the ShardWriter instance unless we get swap our
+        // Arc implementation for a RwLock implementation because we'd need
+        // to write the cache on modification
+
+        let filepath = self.path.join(GENERATION_FILE);
+        // check if file does not exist
+        if !filepath.exists() {
+            return self.new_generation_id();
+        }
+        std::fs::read_to_string(filepath).unwrap()
+    }
+
+    #[tracing::instrument(skip_all)]
+    pub fn new_generation_id(&self) -> String {
+        let generation_id = uuid::Uuid::new_v4().to_string();
+        self.set_generation_id(generation_id.clone());
+        generation_id
+    }
+
+    pub fn set_generation_id(&self, generation_id: String) {
+        let filepath = self.path.join(GENERATION_FILE);
+        std::fs::write(filepath, generation_id).unwrap();
+    }
+
+    pub fn get_kbid(&self) -> String {
+        self.metadata.kbid().unwrap_or_default().to_string()
+    }
+
+    pub fn get_similarity(&self) -> Similarity {
+        self.metadata.similarity().into()
+    }
+
+    pub fn get_shard_segments(&self) -> NodeResult<HashMap<String, Vec<String>>> {
+        let mut segments = HashMap::new();
+
+        segments.insert(
+            "paragraph".to_string(),
+            paragraph_read(&self.paragraph_writer).get_segment_ids()?,
+        );
+        segments.insert(
+            "text".to_string(),
+            text_read(&self.text_writer).get_segment_ids()?,
+        );
+        segments.insert(
+            "vector".to_string(),
+            vector_read(&self.vector_writer).get_segment_ids()?,
+        );
+
+        Ok(segments)
+    }
+
+    pub fn get_shard_files(
+        &self,
+        ignored_segement_ids: &HashMap<String, Vec<String>>,
+    ) -> NodeResult<Vec<IndexFiles>> {
+        let mut files = Vec::new();
+        let _lock = self.write_lock.blocking_lock(); // need to make sure more writes don't happen while we are reading
+
+        files.push(
+            paragraph_read(&self.paragraph_writer)
+                .get_index_files(ignored_segement_ids.get("paragraph").unwrap_or(&Vec::new()))?,
+        );
+        files.push(
+            text_read(&self.text_writer)
+                .get_index_files(ignored_segement_ids.get("text").unwrap_or(&Vec::new()))?,
+        );
+        files.push(
+            vector_read(&self.vector_writer)
+                .get_index_files(ignored_segement_ids.get("vector").unwrap_or(&Vec::new()))?,
+        );
+
+        Ok(files)
     }
 }
