@@ -27,6 +27,7 @@ use nucliadb_core::protos::prost::Message;
 use nucliadb_core::protos::resource::ResourceStatus;
 use nucliadb_core::protos::{Resource, ResourceId};
 use nucliadb_core::tracing::{self, *};
+use nucliadb_core::IndexFiles;
 use nucliadb_procs::measure;
 use regex::Regex;
 use tantivy::collector::Count;
@@ -41,12 +42,11 @@ lazy_static::lazy_static! {
     static ref REGEX: Regex = Regex::new(r"\\[a-zA-Z1-9]").unwrap();
 }
 
-const MERGE_THRESHOLD: usize = 5;
-
 pub struct ParagraphWriterService {
     pub index: Index,
     pub schema: ParagraphSchema,
     writer: IndexWriter,
+    config: ParagraphConfig,
 }
 
 impl Debug for ParagraphWriterService {
@@ -143,24 +143,48 @@ impl WriterChild for ParagraphWriterService {
         Ok(())
     }
 
-    #[measure(actor = "paragraphs", metric = "garbage_collection")]
     #[tracing::instrument(skip_all)]
     fn garbage_collection(&mut self) -> NodeResult<()> {
-        self.writer.garbage_collect_files()?;
         Ok(())
     }
+    fn get_segment_ids(&self) -> NodeResult<Vec<String>> {
+        Ok(self
+            .index
+            .searchable_segment_ids()?
+            .iter()
+            .map(|s| s.uuid_string())
+            .collect())
+    }
 
-    #[measure(actor = "paragraphs", metric = "merge")]
-    #[tracing::instrument(skip_all)]
-    fn merge(&mut self) -> NodeResult<()> {
-        let mut ids = self.index.searchable_segment_ids()?;
-        if ids.len() <= MERGE_THRESHOLD {
-            Ok(())
-        } else {
-            ids.truncate(MERGE_THRESHOLD);
-            self.writer.merge(&ids)?;
-            Ok(())
+    fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles> {
+        // Should be called along with a lock at a higher level to be safe
+        let mut meta_files = HashMap::new();
+        let path = self.config.path.join("meta.json");
+        meta_files.insert("paragraph/meta.json".to_string(), fs::read(path)?);
+
+        let mut files = Vec::new();
+
+        for segment_meta in self.index.searchable_segment_metas()? {
+            if ignored_segment_ids.contains(&segment_meta.id().uuid_string()) {
+                continue;
+            }
+            for seg_file in segment_meta.list_files() {
+                files.push(format!("paragraph/{}", seg_file.to_string_lossy()));
+            }
         }
+
+        if files.is_empty() {
+            // exit with no changes
+            return Ok(IndexFiles {
+                metadata_files: HashMap::new(),
+                files,
+            });
+        }
+
+        Ok(IndexFiles {
+            metadata_files: meta_files,
+            files,
+        })
     }
 }
 
@@ -197,14 +221,15 @@ impl ParagraphWriterService {
         };
 
         index_builder = index_builder.settings(settings);
-        let index = index_builder.create_in_dir(&config.path).unwrap();
+        let index = index_builder.create_in_dir(&config.path)?;
 
-        let writer = index.writer_with_num_threads(1, 6_000_000).unwrap();
+        let writer = index.writer_with_num_threads(1, 6_000_000)?;
 
         Ok(ParagraphWriterService {
             index,
             writer,
             schema: paragraph_schema,
+            config: config.clone(),
         })
     }
     #[tracing::instrument(skip_all)]
@@ -213,12 +238,13 @@ impl ParagraphWriterService {
 
         let index = Index::open_in_dir(&config.path)?;
 
-        let writer = index.writer_with_num_threads(1, 6_000_000).unwrap();
+        let writer = index.writer_with_num_threads(1, 6_000_000)?;
 
         Ok(ParagraphWriterService {
             index,
             writer,
             schema: paragraph_schema,
+            config: config.clone(),
         })
     }
 
@@ -302,9 +328,9 @@ impl ParagraphWriterService {
                 doc.add_u64(self.schema.index, index);
                 doc.add_text(self.schema.split, split);
                 self.writer.delete_term(paragraph_term);
-                self.writer.add_document(doc).unwrap();
+                self.writer.add_document(doc)?;
                 if paragraph_counter % 500 == 0 {
-                    self.writer.commit().unwrap();
+                    self.writer.commit()?;
                 }
             }
         }
