@@ -405,17 +405,34 @@ pub struct FieldMetadata {
     pub relations: ::prost::alloc::vec::Vec<Relations>,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
-pub struct QuestionAnswer {
+pub struct Question {
     #[prost(string, tag="1")]
-    pub question: ::prost::alloc::string::String,
+    pub text: ::prost::alloc::string::String,
     #[prost(string, tag="2")]
-    pub question_language: ::prost::alloc::string::String,
-    #[prost(string, tag="3")]
-    pub answer: ::prost::alloc::string::String,
-    #[prost(string, tag="4")]
-    pub answer_language: ::prost::alloc::string::String,
-    #[prost(int64, repeated, tag="5")]
-    pub paragraph_ids: ::prost::alloc::vec::Vec<i64>,
+    pub language: ::prost::alloc::string::String,
+    #[prost(string, repeated, tag="3")]
+    pub ids_paragraphs: ::prost::alloc::vec::Vec<::prost::alloc::string::String>,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct Answers {
+    #[prost(string, tag="1")]
+    pub text: ::prost::alloc::string::String,
+    #[prost(string, tag="2")]
+    pub language: ::prost::alloc::string::String,
+    #[prost(string, repeated, tag="3")]
+    pub ids_paragraphs: ::prost::alloc::vec::Vec<::prost::alloc::string::String>,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct QuestionAnswer {
+    #[prost(message, optional, tag="1")]
+    pub question: ::core::option::Option<Question>,
+    #[prost(message, repeated, tag="2")]
+    pub answers: ::prost::alloc::vec::Vec<Answers>,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct QuestionAnswers {
+    #[prost(message, repeated, tag="1")]
+    pub question_answer: ::prost::alloc::vec::Vec<QuestionAnswer>,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
 pub struct FieldQuestionAnswerWrapper {
@@ -429,7 +446,7 @@ pub mod field_question_answer_wrapper {
     #[derive(Clone, PartialEq, ::prost::Oneof)]
     pub enum FileOrData {
         #[prost(message, tag="1")]
-        QuestionAnswer(super::QuestionAnswer),
+        QuestionAnswers(super::QuestionAnswers),
         #[prost(message, tag="2")]
         File(super::CloudFile),
     }
