@@ -18,15 +18,17 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 use std::collections::HashMap;
 use std::fmt::Debug;
+use std::fs;
+use std::path::PathBuf;
 use std::time::SystemTime;
 
 use nucliadb_core::metrics::request_time;
-use nucliadb_core::prelude::*;
 use nucliadb_core::protos::prost::Message;
 use nucliadb_core::protos::resource::ResourceStatus;
 use nucliadb_core::protos::{Resource, ResourceId, VectorSetId, VectorSimilarity};
 use nucliadb_core::tracing::{self, *};
 use nucliadb_core::{metrics, Channel};
+use nucliadb_core::{prelude::*, IndexFiles};
 use nucliadb_procs::measure;
 
 use crate::data_point::{DataPoint, Elem, LabelDictionary};
@@ -43,6 +45,7 @@ pub struct VectorWriterService {
     index: Index,
     indexset: IndexSet,
     channel: Channel,
+    config: VectorConfig,
 }
 
 impl Debug for VectorWriterService {
@@ -321,17 +324,104 @@ impl WriterChild for VectorWriterService {
     #[measure(actor = "vectors", metric = "garbage_collection")]
     #[tracing::instrument(skip_all)]
     fn garbage_collection(&mut self) -> NodeResult<()> {
-        let lock = self.index.get_elock()?;
+        let time = SystemTime::now();
+
+        let lock = self.index.try_elock()?;
         self.index.collect_garbage(&lock)?;
+
+        let took = time.elapsed().map(|i| i.as_secs_f64()).unwrap_or(f64::NAN);
+        debug!("Garbage collection {took} ms");
+
         Ok(())
     }
 
-    #[measure(actor = "vectors", metric = "merge")]
-    #[tracing::instrument(skip_all)]
-    fn merge(&mut self) -> NodeResult<()> {
-        let lock = self.index.get_slock()?;
-        self.index.merge(&lock)?;
-        Ok(())
+    fn get_segment_ids(&self) -> NodeResult<Vec<String>> {
+        let mut seg_ids = self.get_segment_ids_for_vectorset(&self.index.location)?;
+        let vectorsets = self.list_vectorsets()?;
+        for vs in vectorsets {
+            let vs_seg_ids = self.get_segment_ids_for_vectorset(&self.config.vectorset.join(vs))?;
+            seg_ids.extend(vs_seg_ids);
+        }
+        Ok(seg_ids)
+    }
+
+    fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles> {
+        // Should be called along with a lock at a higher level to be safe
+        let mut meta_files = HashMap::new();
+        meta_files.insert(
+            "vectors/state.bincode".to_string(),
+            fs::read(self.config.path.join("state.bincode"))?,
+        );
+        meta_files.insert(
+            "vectors/metadata.json".to_string(),
+            fs::read(self.config.path.join("metadata.json"))?,
+        );
+
+        let mut files = Vec::new();
+
+        for segment_id in self.get_segment_ids_for_vectorset(&self.index.location)? {
+            if ignored_segment_ids.contains(&segment_id) {
+                continue;
+            }
+            files.push(format!("vectors/{}/index.hnsw", segment_id));
+            files.push(format!("vectors/{}/journal.json", segment_id));
+            files.push(format!("vectors/{}/nodes.kv", segment_id));
+
+            let fst_path = self.index.location.join(format!("{}/fst", segment_id));
+            if fst_path.exists() {
+                files.push(format!("vectors/{}/fst/keys.fst", segment_id));
+                files.push(format!("vectors/{}/fst/labels.fst", segment_id));
+                files.push(format!("vectors/{}/fst/labels.idx", segment_id));
+            }
+        }
+
+        let vectorsets = self.list_vectorsets()?;
+        if !vectorsets.is_empty() {
+            meta_files.insert(
+                "vectorset/state.bincode".to_string(),
+                fs::read(self.config.vectorset.join("state.bincode"))?,
+            );
+            for vs in vectorsets {
+                for segment_id in
+                    self.get_segment_ids_for_vectorset(&self.config.vectorset.join(vs.clone()))?
+                {
+                    if ignored_segment_ids.contains(&segment_id) {
+                        continue;
+                    }
+                    files.push(format!("vectorset/{}/{}/index.hnsw", vs, segment_id));
+                    files.push(format!("vectorset/{}/{}/journal.json", vs, segment_id));
+                    files.push(format!("vectorset/{}/{}/nodes.kv", vs, segment_id));
+
+                    let fst_path = self.config.vectorset.join(format!("{}/fst", segment_id));
+                    if fst_path.exists() {
+                        files.push(format!("vectors/{}/fst/keys.fst", segment_id));
+                        files.push(format!("vectors/{}/fst/labels.fst", segment_id));
+                        files.push(format!("vectors/{}/fst/labels.idx", segment_id));
+                    }
+                }
+                meta_files.insert(
+                    format!("vectorset/{}/state.bincode", vs),
+                    fs::read(self.config.vectorset.join(format!("{}/state.bincode", vs)))?,
+                );
+                meta_files.insert(
+                    format!("vectorset/{}/metadata.json", vs),
+                    fs::read(self.config.vectorset.join(format!("{}/metadata.json", vs)))?,
+                );
+            }
+        }
+
+        if files.is_empty() {
+            // exit with no changes
+            return Ok(IndexFiles {
+                metadata_files: HashMap::new(),
+                files,
+            });
+        }
+
+        Ok(IndexFiles {
+            metadata_files: meta_files,
+            files,
+        })
     }
 }
 
@@ -383,6 +473,7 @@ impl VectorWriterService {
                 )?,
                 indexset: IndexSet::new(indexset)?,
                 channel: config.channel,
+                config: config.clone(),
             })
         }
     }
@@ -398,9 +489,24 @@ impl VectorWriterService {
                 index: Index::open(path)?,
                 indexset: IndexSet::new(indexset)?,
                 channel: config.channel,
+                config: config.clone(),
             })
         }
     }
+
+    fn get_segment_ids_for_vectorset(&self, location: &PathBuf) -> NodeResult<Vec<String>> {
+        let mut ids = Vec::new();
+        for dir_entry in std::fs::read_dir(location)? {
+            let entry = dir_entry?;
+            let path = entry.path();
+            let name = entry.file_name().to_string_lossy().to_string();
+            if path.is_file() {
+                continue;
+            }
+            ids.push(name);
+        }
+        Ok(ids)
+    }
 }
 
 #[cfg(test)]
@@ -573,4 +679,82 @@ mod tests {
         let res = writer.set_resource(&resource);
         assert!(res.is_ok());
     }
+
+    #[test]
+    fn test_get_segments() {
+        let dir = TempDir::new().unwrap();
+        let vsc = VectorConfig {
+            similarity: Some(VectorSimilarity::Cosine),
+            path: dir.path().join("vectors"),
+            vectorset: dir.path().join("vectorset"),
+            channel: Channel::EXPERIMENTAL,
+        };
+        let resource_id = ResourceId {
+            shard_id: "DOC".to_string(),
+            uuid: "DOC/KEY".to_string(),
+        };
+
+        let mut sentences = HashMap::new();
+        sentences.insert(
+            "DOC/KEY/1/1".to_string(),
+            VectorSentence {
+                vector: vec![1.0, 3.0, 4.0],
+                ..Default::default()
+            },
+        );
+
+        let paragraph = IndexParagraph {
+            start: 0,
+            end: 0,
+            sentences,
+            field: "".to_string(),
+            labels: vec!["1".to_string(), "2".to_string(), "3".to_string()],
+            index: 3,
+            split: "".to_string(),
+            repeated_in_field: false,
+            metadata: None,
+        };
+        let paragraphs = IndexParagraphs {
+            paragraphs: HashMap::from([("DOC/KEY/1".to_string(), paragraph)]),
+        };
+        let resource = Resource {
+            resource: Some(resource_id.clone()),
+            metadata: None,
+            texts: HashMap::with_capacity(0),
+            status: ResourceStatus::Processed as i32,
+            labels: vec!["FULL".to_string()],
+            paragraphs: HashMap::from([("DOC/KEY".to_string(), paragraphs)]),
+            paragraphs_to_delete: vec![],
+            sentences_to_delete: vec![],
+            relations_to_delete: vec![],
+            relations: vec![],
+            vectors: HashMap::default(),
+            vectors_to_delete: HashMap::default(),
+            shard_id: "DOC".to_string(),
+        };
+        // insert - delete - insert sequence
+        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let res = writer.set_resource(&resource);
+        assert!(res.is_ok());
+        let res = writer.delete_resource(&resource_id);
+        assert!(res.is_ok());
+        let res = writer.set_resource(&resource);
+        assert!(res.is_ok());
+
+        let segments = writer.get_segment_ids().unwrap();
+        assert_eq!(segments.len(), 2);
+        let existing_segs: Vec<String> = Vec::new();
+        let index_files = writer.get_index_files(&existing_segs).unwrap();
+        let mut expected_files = Vec::new();
+        for segment in segments {
+            expected_files.push(format!("vectors/{}/index.hnsw", segment));
+            expected_files.push(format!("vectors/{}/journal.json", segment));
+            expected_files.push(format!("vectors/{}/nodes.kv", segment));
+            expected_files.push(format!("vectors/{}/fst/keys.fst", segment));
+            expected_files.push(format!("vectors/{}/fst/labels.fst", segment));
+            expected_files.push(format!("vectors/{}/fst/labels.idx", segment));
+        }
+        assert_eq!(index_files.files, expected_files);
+        assert_eq!(index_files.metadata_files.len(), 2);
+    }
 }
