@@ -22,7 +22,6 @@ use std::io::Cursor;
 use std::sync::Arc;
 
 use nucliadb_core::protos::*;
-use nucliadb_core::tracing;
 use nucliadb_node::shards::metadata::ShardMetadata;
 use nucliadb_node::shards::providers::unbounded_cache::UnboundedShardWriterCache;
 use nucliadb_node::shards::providers::ShardWriterProvider;
@@ -39,6 +38,7 @@ use crate::errors::{IndexNodeException, LoadShardError};
 use crate::RawProtos;
 
 #[pyclass]
+#[derive(Default)]
 pub struct NodeWriter {
     shards: UnboundedShardWriterCache,
 }
@@ -62,13 +62,13 @@ impl NodeWriter {
 impl NodeWriter {
     #[new]
     pub fn new() -> PyResult<Self> {
-        let data_path = env::data_path();
-        let shards_path = env::shards_path();
-        lifecycle::initialize_writer(&data_path, &shards_path)
-            .map_err(|error| format!("Unable to initialize writer: {error}"))
-            .map_err(IndexNodeException::new_err)?;
+        if let Err(error) = lifecycle::initialize_writer(&env::data_path(), &env::shards_path()) {
+            return Err(IndexNodeException::new_err(format!(
+                "Unable to initialize writer: {error}"
+            )));
+        };
         Ok(Self {
-            shards: UnboundedShardWriterCache::new(shards_path),
+            shards: UnboundedShardWriterCache::new(env::shards_path()),
         })
     }
 
@@ -87,7 +87,7 @@ impl NodeWriter {
                     paragraph_service: new_shard.paragraph_version() as i32,
                     vector_service: new_shard.vector_version() as i32,
                     relation_service: new_shard.relation_version() as i32,
-                    id: new_shard.id,
+                    id: new_shard.id.clone(),
                 }
                 .encode_to_vec(),
             )),
@@ -145,12 +145,8 @@ impl NodeWriter {
         let status = shard
             .set_resource(&resource)
             .and_then(|()| shard.get_opstatus());
-
         match status {
             Ok(mut status) => {
-                if let Err(err) = shard.merge() {
-                    tracing::info!("A merge was attempted, but failed: {err:?}")
-                }
                 status.status = 0;
                 status.detail = "Success!".to_string();
                 Ok(PyList::new(py, status.encode_to_vec()))
@@ -182,9 +178,6 @@ impl NodeWriter {
             .and_then(|()| shard.get_opstatus());
         match status {
             Ok(mut status) => {
-                if let Err(err) = shard.merge() {
-                    tracing::info!("A merge was attempted, but failed: {err:?}")
-                }
                 status.status = 0;
                 status.detail = "Success!".to_string();
                 Ok(PyList::new(py, status.encode_to_vec()))
